import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.Image;

import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;
import java.io.File;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Random;
import javax.imageio.ImageIO;

import static java.lang.Math.*;

public class AffineFractal implements Runnable {
	public AffineFractal (String...args) {}

	static final int PAD = 20;

	@Override
	@SuppressWarnings("unchecked")
	public void run () {
		final JFrame frame = new JFrame("AffineFractal");
		final BufferedImage baseImage;

		try {
			baseImage = ImageIO.read(new File("images/image.png"));
		} catch (Exception ex) {
			ex.printStackTrace();
			return;
		}

		final List<AffineTransform> transforms = new ArrayList<AffineTransform>();

		for (int i = 0; i < 6; ++i) {
			final double theta = (2 * PI * i) / 6;
			AffineTransform trans = AffineTransform.getScaleInstance(0.4, 0.4);
			trans.translate(0.8 * cos(theta), 0.8 * sin(theta));
			transforms.add(trans);
		}

		final int[] maxDepth = {0};
		final int imageWidth  = baseImage.getWidth();
		final int imageHeight = baseImage.getHeight();
		final double imageScale = min(1.0/imageWidth, 1.0/imageHeight);

		final JPanel panel = new JPanel () {
			@Override
			public void paintComponent (Graphics g) {
				Graphics2D graphics = (Graphics2D)g;


				int width = getWidth();
				int height = getHeight();

				graphics.setColor(Color.WHITE);
				graphics.fillRect(0, 0, width, height);

				AffineTransform trans = AffineTransform.getTranslateInstance(0.5*width, 0.5*height);

				double scale = 0.9 * min(height, width);

				trans.scale(scale, scale);

				graphics.setTransform(trans);

				graphics.setStroke(new BasicStroke(0.1f));
				graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				graphics.setColor(Color.GREEN);
				graphics.draw(ellipse);
				graphics.setColor(Color.BLUE);

				drawFractal(trans, graphics, maxDepth[0]);
			}

			final Ellipse2D.Double ellipse = new Ellipse2D.Double (-0.5, -0.5, 1.0, 1.0);

			void drawFractal (AffineTransform trans, Graphics2D graphics, int depth) {
				for (AffineTransform transform : transforms) {
					AffineTransform outputTrans = new AffineTransform(trans);

					outputTrans.concatenate(transform);

					if (depth > 0) {
						drawFractal(outputTrans, graphics, depth - 1);
					} else {
						outputTrans.scale(imageScale, imageScale);

						graphics.setTransform(outputTrans);

						graphics.drawImage(baseImage, -imageWidth/2, -imageHeight/2, imageWidth, imageHeight, null);
						//~ graphics.draw(ellipse);
					}
				}
			}
		};

		new Timer(750, new ActionListener() {
			@Override
			public void actionPerformed (ActionEvent event) {
				maxDepth[0] = (maxDepth[0] + 1) % 6;

				panel.repaint();
			}
		}).start();

		frame.setLayout(new BorderLayout());
		frame.add(panel, BorderLayout.CENTER);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
		frame.setSize(800, 600);
	}

	public static void main (String...args) {
		SwingUtilities.invokeLater(new AffineFractal(args));
	}
}
